# VSCode Autocomplete for Meld - Implementation Plan

This document outlines a pragmatic plan for implementing a VSCode extension that provides autocomplete functionality for Meld scripts, leveraging the existing meld-ast library.

## Goals

1. Provide intelligent autocompletion for Meld directives, variables, and paths
2. Support syntax highlighting for Meld files (`.meld`, `.meld.md`, `.mll`, `.mll.md`)
3. Offer contextual suggestions based on the current cursor position
4. Provide hover documentation for Meld directives and syntax
5. Detect and highlight syntax errors in real-time

## Architecture Overview

The VSCode extension will be built using VSCode's native extension APIs, without the complexity of a Language Server Protocol implementation. It will:

1. Use **meld-ast** directly for parsing and analysis
2. Implement VSCode's completion provider interface 
3. Track variables and imports in the current document
4. Provide context-aware completions based on cursor position

## Implementation Plan

### Phase 1: Basic VSCode Extension (1 week)

1. Create a new VSCode extension project
2. Set up language configuration for Meld
3. Define syntax highlighting using TextMate grammar
4. Configure extension to activate for Meld file types (`.meld`, `.meld.md`, `.mll`, `.mll.md`)

### Phase 2: Parser Integration (1 week)

1. Import meld-ast library as a dependency
2. Create a simple document parser service
3. Extract variables, commands, and paths from the AST
4. Implement error reporting from parser errors to VSCode diagnostics

### Phase 3: Core Completions (1 week)

Implement completion providers that trigger on specific patterns:

1. After `@` → Show directive completions (@text, @data, @path, etc.)
2. After `${` → Show text variable completions
3. After `#{` → Show data variable completions
4. After `$` (but not `${`) → Show path variable completions
5. Inside `@run [$...` → Show command completions

### Phase 4: Enhanced Features (1-2 weeks, optional)

Add more advanced features as time allows:

1. Hover information for directives and variables
2. Path completion for @embed and @import directives
3. Basic "Go to Definition" for variables
4. Simple snippets for common Meld patterns

## Technical Approach

### Using meld-ast Efficiently

The extension will use meld-ast in a straightforward way:

```typescript
// When completion is requested:
function provideCompletions(document, position) {
  // Parse the document
  const text = document.getText();
  const ast = meldAst.parse(text);
  
  // Extract variables, paths, commands from AST
  const variables = getVariablesFromAst(ast);
  
  // Determine what kind of completion is needed based on cursor position
  const context = getCompletionContext(document, position);
  
  // Return appropriate completions
  if (context.type === 'directive') {
    return getDirectiveCompletions(context.prefix);
  } else if (context.type === 'textVariable') {
    return getTextVariableCompletions(variables, context.prefix);
  } // etc.
}
```

### Context Detection

The extension will detect context by examining text around the cursor:

1. Check if cursor is at `@` → directive context
2. Check if cursor is after `${` → text variable context
3. Check if cursor is after `#{` → data variable context
4. Check for other patterns to determine the appropriate completion type

### Optimizations

1. Parse only when necessary (on document change or completion request)
2. Cache parsed results until document changes
3. Implement simple debouncing for diagnostics updates
4. Focus on completing local variables first, then handle imports if needed

## Key Features

1. **Directive Completion**: Show available directives after typing `@`
2. **Variable Completion**: Complete variable names after `${`, `#{`, or `$`
3. **Basic Hover Info**: Show brief documentation when hovering over directives
4. **Syntax Highlighting**: Color-code Meld constructs for better readability
5. **Error Highlighting**: Underline syntax errors and provide error messages

## Future Enhancements (Post-Initial Release)

1. Import resolution for variable completions from imported files
2. Command parameter completion
3. Integration with filesystem for path completions
4. Smart indent rules for Meld constructs
5. Code folding for multi-line blocks

## Development Timeline

A focused implementation could follow this timeline:

1. Phase 1 (Basic Setup): 3-5 days
2. Phase 2 (Parser Integration): 2-4 days
3. Phase 3 (Core Completions): 3-5 days
4. Phase 4 (Enhanced Features): 5-10 days (optional)

**Total Development Time: 2-3 weeks** for a solid initial version with core functionality.

## Distribution

1. Package as a VSCode extension with meld-ast as a dependency
2. Bundle minimal documentation for directives
3. Publish to VS Code Marketplace with appropriate tags
4. Set up a release process that follows the main Meld codebase versions